/// \file	conf.cpp
///	\brief	Config class source

#include "conf.h"
#include "colorset.h"

using namespace std;
using namespace strutil;
using namespace MLS;

Config MLS::e_tConf;	///<	config global 변수

///	\brief	생성자
Config::Config()
{	
}

///	\brief	소멸자
Config::~Config()
{
}

bool Config::Init()
{
	// . 기본 멤버 변수를 초기화 한다.
	set_default();
	return true;
}	

/// \brief	기본 멤버 변수를 초기화 한다.
void Config::set_default()
{// default를 만듭니다..
	struct stat 	  stat_bufs;	

	// . home 디렉토리 지정
	struct passwd *pw = getpwuid(getuid()); // getuid이냐 geteuid냐...;
	home  = pw->pw_dir;
	home += '/';
	
	// . config dir 지정
	cfghome = home + ".mls/";
	
	// .config dir이 없으면 config dir 생성
	if (lstat (cfghome.c_str(), &stat_bufs) == -1)
	{
		cout << "Making mls configuration directory..." << endl;
		
		if (mkdir(cfghome.c_str(), 0700) == -1) 
		{
			cout << "Making directory operation failed..." << endl;
			exit(-1);
		}		
	}	
	
	// 모든 엔트리를 초기화 시킨다.
	
	// . 펑션 테이블 클리어	
	for (int t=0; t<12; ++t) Func[t].clear();	
	for (int t=0; t<12; ++t) McdFunc[t].clear();	
	
	// 문자형	
	colorset = "mls.col";
	date_type = "%y-%m-%d";
	extern_viewer = "vi";
	extern_editor = "vi";
	mcdroot = home;
	
	// On/Off 형	
	FixedRow = false;
	DirSort  = true;
	ShowHidden  = false;
	McdSort  = true;
	QMlsExit = false;
	QMcdExit = false;
		
	// 정수형
	sort   = 1;
	column = 1;	
	AutoDrawmode = true;	
	
}

///	\brief	mls.cfg 설정 파일을 읽는다.
///	\param	file	mls.cfg config file name
///	\return	0	: 읽기 성공\n
///			1	: 읽기 실패
int Config::load(const char *file)
{
	char line[1024];	
	string::size_type p;
	
	map<string, string> cmd;
	string l, var, val;
	
	// . 파일을 연다.
	FILE *fp=fopen(file, "rb");
	
	if (!fp) return ERROR;
	
	// . 파일의 끝이 아닐때까지 반복
	while(!feof(fp))
	{
		// .. 한 줄을 얻음
		line[0]=0;
		fgets(line, 1024, fp);

		// .. 주석이면 다음으로 넘어감
		if (line[0] == '#') continue;
		
		l=line;
		
		// .. = 을 찾는다.
		p = l.find('=');
				
		if (p == string::npos) continue;
		
		// . 변수와 값을 찾는다.
		var = chop(l.substr(0, p));
		val = chop(l.substr(p+1));
		
		if (val.empty()) continue;
		
		if (var.substr(0, 3) == "ext")
		{
			StringTokenizer st(getbetween(var, '[', ']'), "; ");
				
			Action pr;
			pr.addEntry(val);
						
			while (st.Next())
				ext_prog[tolower(st.Get())] = pr;
		}
		else cmd[tolower(var)]= val; // 나머지들..
	}
	
	fclose(fp);
	
	map<string,string>::iterator i, end = cmd.end();
	
	//set_default();
	
	// . 펑션키	설정
	if ((i = cmd.find("funcf1")) != end) Func[0].addEntry((*i).second);
	if ((i = cmd.find("funcf2")) != end) Func[1].addEntry((*i).second);
	if ((i = cmd.find("funcf3")) != end) Func[2].addEntry((*i).second);
	if ((i = cmd.find("funcf4")) != end) Func[3].addEntry((*i).second);
	if ((i = cmd.find("funcf5")) != end) Func[4].addEntry((*i).second);
	if ((i = cmd.find("funcf6")) != end) Func[5].addEntry((*i).second);
	if ((i = cmd.find("funcf7")) != end) Func[6].addEntry((*i).second);
	if ((i = cmd.find("funcf8")) != end) Func[7].addEntry((*i).second);
	if ((i = cmd.find("funcf9")) != end) Func[8].addEntry((*i).second);
	if ((i = cmd.find("funcf10"))!= end) Func[9].addEntry((*i).second);
	if ((i = cmd.find("funcf11"))!= end) Func[10].addEntry((*i).second);
	if ((i = cmd.find("funcf12"))!= end) Func[11].addEntry((*i).second);
	
	if ((i = cmd.find("mcdfuncf1")) != end) McdFunc[0].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf2")) != end) McdFunc[1].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf3")) != end) McdFunc[2].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf4")) != end) McdFunc[3].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf5")) != end) McdFunc[4].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf6")) != end) McdFunc[5].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf7")) != end) McdFunc[6].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf8")) != end) McdFunc[7].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf9")) != end) McdFunc[8].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf10"))!= end) McdFunc[9].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf11"))!= end) McdFunc[10].addEntry((*i).second);
	if ((i = cmd.find("mcdfuncf12"))!= end) McdFunc[11].addEntry((*i).second);
	
	if ((i = cmd.find("mcdzipfuncf1")) != end) McdZipFunc[0].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf2")) != end) McdZipFunc[1].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf3")) != end) McdZipFunc[2].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf4")) != end) McdZipFunc[3].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf5")) != end) McdZipFunc[4].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf6")) != end) McdZipFunc[5].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf7")) != end) McdZipFunc[6].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf8")) != end) McdZipFunc[7].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf9")) != end) McdZipFunc[8].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf10"))!= end) McdZipFunc[9].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf11"))!= end) McdZipFunc[10].addEntry((*i).second);
	if ((i = cmd.find("mcdzipfuncf12"))!= end) McdZipFunc[11].addEntry((*i).second);
	
	// . 문자형	변수 초기화
	if ((i = cmd.find("colorset")) != end )  colorset = (*i).second;
	if ((i = cmd.find("date_type")) != end ) date_type = (*i).second;
	if ((i = cmd.find("extviewer")) != end ) extern_viewer = (*i).second;
	if ((i = cmd.find("exteditor")) != end ) extern_editor = (*i).second;	
	if ((i = cmd.find("mcdroot")) != end ) mcdroot = (*i).second;	
	
	// . On/Off 형 멤버변수 초기화
	if ((i = cmd.find("fixedrow")) != end ) 
		FixedRow = (tolower((*i).second) == "on");
	if ((i = cmd.find("dirsort")) != end )
		DirSort  = (tolower((*i).second) == "on");
	if ((i = cmd.find("showhidden")) != end )
		ShowHidden  = (tolower((*i).second) == "on");
	if ((i = cmd.find("mcdsort")) != end )
		McdSort  = (tolower((*i).second) == "on");
	if ((i = cmd.find("qmlsexit")) != end)
		QMlsExit = (tolower((*i).second) == "on");
	if ((i = cmd.find("qmcdexit")) != end)
		QMcdExit = (tolower((*i).second) == "on");
		
	// . 정수형	멤버 변수 초기화
	if ((i = cmd.find("sort")) != end )   sort = atoi((*i).second.c_str());
	if ((i = cmd.find("column")) != end ) column = atoi((*i).second.c_str());	
	
	AutoDrawmode = column ? false : true;
	
	// 후처리
	// . mcdroot를 지정한다.
	if (mcdroot.empty()) mcdroot = '/';
	else if (mcdroot[mcdroot.size()-1] != '/') mcdroot += '/';	
	
	return SUCCESS;
}

///	\brief	colorset을 load한다.
///	\param	file	colorset file name
///	\return	0	: colorset 읽기 성공\n
///			1	: colorset 읽기 실패
/*int Config::load_colorset(const char *file)
{
	map<string, color_entry> color_set;	
	string line;
	
	// . 파일을 연다.
	ifstream in(file);
	
	if (!in.is_open()) return 1;

	// . 파일을 끝까지 읽음
	while(!in.eof())
	{		
		// . 한줄을 읽음
		getline(in, line);
		
		// . 주석처리
		if (line[0] == '#') continue;
		
		// . = 을 찾는다		
		string::size_type p = line.find('=');
		
		if (p == string::npos) continue;
		
		// . var와 값을 찾는다.
		string var = tolower(chop(line.substr(0, p)));
		string val = tolower(chop(line.substr(p+1)));	
		
		// . mask[]
		if (var.substr(0, 4) == "mask")
			mask_col[strtol(getbetween(var, '[', ']').c_str(), NULL, 8)] = color_entry(val);
		else if(var.substr(0, 4) == "name")
		{
			StringTokenizer st(getbetween(var, '[', ']'), ";");
			
			color_entry entry(val);
			
			while(st.Next()) name_col[st.Get()] = entry;
		}		
		// ext[]		
		else if (var.substr(0, 3) == "ext")
		{
			StringTokenizer st(getbetween(var, '[', ']'), ";");
			
			color_entry entry(val);
			LOG("val : %s", val.c_str());
			
			while(st.Next())
			{
				LOG("exts : %s", st.Get().c_str());
				if (st.Get().empty()) continue;
				ext_col[st.Get()] = entry;
			}
		}
		// 이외것들은 컬러셋
		else color_set[var] = color_entry(val);
	}
	
	MCD = color_set["mcd"];
	MCDLine = color_set["mcdline"];
	MCDstat = color_set["mcdstat"];
	Dir = color_set["dir"];
	Drive = color_set["drive"];
	Line = color_set["line"]; 
	FuncN = color_set["funcn"];
	FuncA = color_set["funca"];
	StatN = color_set["statn"];
	StatA = color_set["stata"];
	Else = color_set["else"];
	Help = color_set["help"];
	Quick = color_set["quick"];
	QuickMCD = color_set["quickmcd"];
	Dialog = color_set["dialog"];
	
	return 0;
}
*/
