///	@file	dialog.cpp
///	@brief	dialog.cpp
#include <string.h>
#include <stdarg.h>

#include "dialog.h"
#include "mlslocale.h"
#include "drawutil.h"
#include "keydef.h"
#include "strutil.h"

using namespace strutil;
using namespace MLS;
using namespace std;

///	@brief	메세지 박스를 그린다	
///	@param	b_c		배경 색상
///	@param	title	title message
///	@param	msg		출력할 글자
///	@return	성공시 1을 반환 
int MLS::MsgBox(int b_c, const char *title, const char *msg,...)
{
	va_list args;
    char szBuf[256];

    va_start(args,msg);
    vsprintf(szBuf,msg,args);
    va_end(args);
    
    return MsgBox(title, szBuf, b_c);
}


///	@brief	메세지 박스를 그린다	
///	@param	title	title message
///	@param	msg		출력할 글자
///	@param	back_color	배경 색상
///	@return	성공시 1을 반환 
int MLS::MsgBox(const char *title, const char *msg, int back_color)
{	
	int l_title = krstrlen(title);
	int l_msg = krstrlen(msg);
	int width = l_title > l_msg ? l_title : l_msg;
	
	width +=4;
			
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	wclear(win);
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	
	wattron(win ,A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	// msg 출력
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	mvwprintw(win, 3, (width - l_msg)/2, "%s", msg);
	wattroff(win, A_BOLD);
	
	wrefresh(win);
	
	getch();	
	delwin(win);
	return 1;	
}

///	@brief	메세지 박스를 그린다	
///	@param	title	title message
///	@param	msg		입력받은 글자[반환값]
///	@param	back_color	배경 색상
///	@return	메세지 박스 WINDOW
WINDOW* MLS::MsgWaitBox(const char *title, const char *msg, int back_color)
{
	int l_title = krstrlen(title);
	int l_msg = krstrlen(msg);
	int width = l_title > l_msg ? l_title : l_msg;
	
	width +=4;
			
	// . width,5[x,y] 크기의 박스를 만든다.
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	
	wattron(win ,A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	// msg
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	mvwprintw(win, 3, (width - l_msg)/2, "%s", msg);
	wattroff(win, A_BOLD);
	
	wrefresh(win);
	return win;
}

///	@brief	해당 Window를 제거한다.
///	@param	win		제거할 window
///	@return	1을 반환
int MLS::MsgWaitEnd(WINDOW*	 win)
{
	wclear(win);
	wrefresh(win);
	delwin(win);
	refresh();
	return 1;	
}

///	@brief	Input Box를 그린다.
///	@param	title	title message
///	@param	str		입력받은 글자[반환값]
///	@param	back_color	배경 색상
///	@return	1 : 성공\n -1 : 실패
int MLS::InputBox(const char *title, std::string &str, int back_color)
{ // 글자가 넘어갈 경우 스크롤시킨다.
	
	int l_title = krstrlen(title);	
	int width = COLS - 4;
	int W = width - 4;
	curs_set(1); // 커서를 보이게 한다.
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
		
	wattron(win ,A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	int cur=str.size(), scroll= str.size() > W ? str.size()-W : 0; // 커서, 스크롤
	
	int key;
	
	while(1)
	{
		// 찍고
		wattron (win, COLOR(COLOR_WHITE, back_color));
		wattron (win, A_BOLD);
		wmove(win, 3, 2);
		whline(win, ' ', W);
		wprintw(win, "%s", str.substr(scroll, W).c_str());
		wattroff(win, A_BOLD);
		
		// 커서가 있는 곳으로 이동한다.
		wmove(win, 3, 2+cur-scroll);
		
		wrefresh(win);
		
		key = getch();
		
		// home, end, pgup, pgdn, 화살표, 아스키
		// 기타 키가 들어오면 씹는다..
		if (key == KEY_HOME || key == KEY_PPAGE || key == KEY_UP)
		{
			cur = 0;
			scroll = 0;
		}
		
		if (key == KEY_END || key == KEY_NPAGE || key == KEY_DOWN)  
		{
			cur = str.size();
			if (cur - scroll >= W) scroll = cur - W ;
		}
		
		if (key == KEY_LEFT)
		{
			cur--;
			if (cur < 0) cur = 0;
			if (cur - scroll < 0) scroll = cur;
		}
		
		if (key == KEY_RIGHT)
		{
			cur++;
			if (cur > str.size()) cur = str.size();
			if (cur - scroll >= W) scroll = cur - W ;
		}
						
		if (key == 27) { curs_set(1); delwin(win); return -1; }
		if (key == 13) break;
		
		// backspace 커서 위치에서 한글자 지운다.
		if (key == 127)
		{
			if (!str.empty() && cur)	
			{
				str.erase(cur-1, 1);
				cur--;
				if (cur - scroll < 0) scroll = cur;
			}			
			continue;
		}
		
		// del..		
		if (key == 330)
		{
			if (!str.empty())
				str.erase(cur, 1);
			continue;
		}
		
		if (32 <= key && key <= 127)
		{
			//MsgBox(1, "f", "%d", cur);
			str.insert(cur, 1, (char) key);
			cur++;
			if (cur - scroll >= W) scroll = cur - W;
			continue;
		}
	}
	delwin(win);
	curs_set(0); // // 커서를 보이지 않게 한다.
	return 1;	
}

///	@brief	Select Box를 그린다.
///	@param	title	title message
///	@param	p		선택할 수 있는 글자들[화면에 출력할 글자]
///	@param	sel		기본으로 sel번째 글자가 선택됨
///	@param	back_color	배경 색상
///	@return	0 이상 : 선택된 글자 index\n -1 : 실패
int MLS::SelectBox(const char *title, vector<string> &p, int sel, int back_color)
{
	typedef vector<string>::iterator Itor;
	vector<int> location;
	
	// . 화면에 선택 박스는 최소한 2개이상의 선택할수 있는 것이 있어야함
	if (p.size() < 2) return -1;
	
	// . 화면에 출력할 글자들의 위치를 구한다.
	location.push_back(3);	
	for (Itor i=p.begin(); i!=p.end()-1; ++i)
		location.push_back( location.back() + (*i).size() + 3);
	
	// . 화면에 출력할 박스 크기를 구한다.[좌우3크기의 여백을 가진다.]
	int width = location.back() + p.back().size() + 3;
	if (width < krstrlen(title) + 4) width = krstrlen(title) + 4;
	
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	wattron(win, A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	setcol(COLOR_BLACK, COLOR_WHITE, win);
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - krstrlen(title))/2 , "%s", title);
	unsetcol(win);
	
	int key;
	
	//--Yes--- ---No---
	while(1)
	{// 출력
		wmove (win, 3, (width-17)/2);
		
		setcol(COLOR_WHITE, back_color, win);		
		for (int t=0; t<p.size() ;++t)
		{
			wmove(win, 3, location[t]-1);
			wprintw(win, " %s ", p[t].c_str());
		}
		unsetcol();
		
		setcol(back_color, COLOR_WHITE, win);
		wmove(win, 3, location[sel]-1);
		wprintw(win, " %s ", p[sel].c_str());
		unsetcol();
		
		wmove(win, 0, 0);
		
		wrefresh(win);		
		
		key=getch();
		
		if (key == 13) break;
		
		if (key == 27 || key == '`')
		{
			delwin(win);
			return -1;
		}
		
		switch(key)
		{			
		case '\t':
			sel = (sel + 1) % p.size();
			break;
			
		case KEY_UP:
			sel = 0;
			break;
						
		case KEY_DOWN:
			sel = p.size() - 1;
			break;
			
		case KEY_LEFT:
			sel = (sel - 1 + p.size()) % p.size();
			break;
			
		case KEY_RIGHT:
			sel = (sel + 1) % p.size();
			break;		
		}						
		
	}	
	delwin(win);
	
	return sel;
		
}

///	@brief	Yes와 No가 있는 Box를 그리고 선택하게 한다.
///	@param	title		Yes No Box title
///	@param	YN			Yes나 No 기본 선택값
///	@param	back_color	배경 색상
/// @return	YN_Y/YN_N
int MLS::YNBox(const char *title, int YN, int back_color)
{
	int l_title = krstrlen(title);
	int width = l_title > 17 ? l_title : 17;	

	width +=4;
			
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	
	wattron(win ,A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	int key;
	
	//--Yes--- ---No---
	while(1)
	{// 출력
		wmove (win, 3, (width-17)/2);
		
		if (YN == YN_Y)
			wattron (win, COLOR(back_color, COLOR_WHITE));
		else 
		{
			wattron (win, COLOR(COLOR_WHITE, back_color));
			wattron (win, A_BOLD);
		}
		
		wprintw(win, strmid(8, gettext("Yes")).c_str());
		wattroff(win, A_BOLD);
		
		wattron (win, COLOR(COLOR_WHITE, back_color));
		wattron (win, A_BOLD);
		waddch(win, ' ');
		wattroff(win, A_BOLD);
		
		if (YN == YN_N)
			wattron (win, COLOR(back_color, COLOR_WHITE));
		else 
		{
			wattron (win, COLOR(COLOR_WHITE, back_color));
			wattron (win, A_BOLD);
		}
									
		wprintw(win, strmid(8, gettext("No")).c_str());
		wattroff(win, A_BOLD);		
		
		wrefresh(win);	
		
		key=getch();
		
		if (key == 13) break;
		
		if (key == 27)
		{
			delwin(win);
			return YN_N;
		}
		
		switch(key)
		{
		case '\t':
		case KEY_UP:
		case KEY_DOWN:
		case KEY_LEFT:
		case KEY_RIGHT:
			if (YN == YN_Y) YN=YN_N;
			else YN = YN_Y;
			break;		
		}						
		
	}	
	delwin(win);
	
	return 	YN;
}

ProgressBox::ProgressBox(const char *_title, const char *_msg, int _back_color)
{
	title = _title;
	msg =_msg;
	back_color = _back_color;
	
	m_nCounter=0;
	
	win = newwin(8, 72, (LINES-8)/2, (COLS-72)/2);
	redraw();
	
	sRightString = "";
	sLeftString = "";
}

///	@brief	상태 Box소멸자
ProgressBox::~ProgressBox()
{
	delwin(win);
}

///	@brief	상태 Box를 그리는 함수
void ProgressBox::redraw()
{
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	
	wattron(win ,A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', 72-2);
	mvwprintw(win, 1, (72 - krstrlen(title))/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	// msg 출력
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	mvwprintw(win, 3, (72 - krstrlen(msg))/2, "%s", msg);
	wattroff(win, A_BOLD);	
}

void ProgressBox::show()
{
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	
	wmove(win, 5, 6);
	whline(win, ' ', 60);
	if (sLeftString != "") mvwprintw(win, 5, 6, "%s", sLeftString.c_str());
	if (sRightString != "") mvwprintw(win, 5, 72 - sRightString.size() -6, "%s", sRightString.c_str());
	wattroff(win, A_BOLD);
	
	//U, B 기본바탕
	wattron(win, COLOR(COLOR_WHITE, COLOR_BLACK));
	wattron(win, A_BOLD);
	wmove(win, 6, 6);
	whline(win, '.', 60);
	wattroff(win, A_BOLD);
	
	wattron(win, COLOR(COLOR_WHITE, COLOR_WHITE));	
	wmove(win, 6, 6);
	whline(win, ' ', m_nCounter);
	wattroff(win, A_BOLD);
	wrefresh(win);
}

void ProgressBox::setCount(int nCount) 
{ 
	if (nCount <= 0) 
	{
		m_nCounter = 0;
		return;
	}
	
	if (nCount >= 100) 
	{
		m_nCounter = 60;
		return;
	}
	
	m_nCounter = (int)(((double)nCount / 100.0) * 60.0);
}

///	@brief	상태 Box생성자 
///	@param	_title		title 글자
///	@param	_msg		화면에 출력할 내용
///	@param	_back_color	배경 색상
StatBox::StatBox(const char *_title, const char *_msg, int _back_color)
{
	title = _title;
	msg =_msg;
	back_color = _back_color;
	
	UL = NULL;	
	UR = NULL;
	BL = NULL;
	BR = NULL;
	
	U=0;
	B=0;
	
	win = newwin(12, 72, (LINES-12)/2, (COLS-72)/2);
	redraw();	
}

///	@brief	상태 Box소멸자
StatBox::~StatBox()
{
	delwin(win);
}

///	@brief	상태 Box를 그리는 함수
void StatBox::redraw()
{
	wbkgd(win, COLOR(COLOR_WHITE, back_color));	
	
	wattron(win ,A_BOLD);
	wborder(win, VLINE, VLINE, HLINE, HLINE, ULCORNER, URCORNER, LLCORNER, LRCORNER);
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', 72-2);
	mvwprintw(win, 1, (72 - krstrlen(title))/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	// msg 출력
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	mvwprintw(win, 3, (72 - krstrlen(msg))/2, "%s", msg);
	wattroff(win, A_BOLD);	
}

//-
//타이틀
//
//To c:
//
//|  Autoexec
//----...
//
//<21/132>
//----...
//
//---

void StatBox::show()
{
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	// UL UR 출력
	
	wmove(win, 5, 6);
	whline(win, ' ', 60);
	if (UL) mvwprintw(win, 5, 6, "%s", UL);
	if (UR) mvwprintw(win, 5, 72 - krstrlen(UR) -6, "%s", UR);	
	
	// BL BR 출력
	wmove(win, 8, 6);
	whline(win, ' ', 60);
	if (BL) mvwprintw(win, 8, 6, "%s", BL);
	if (BR) mvwprintw(win, 8, 72 - krstrlen(BR) -6, "%s", BR);	
	
	wattroff(win, A_BOLD);	
	
	//U, B 기본바탕
	wattron(win, COLOR(COLOR_WHITE, COLOR_BLACK));	
	wattron(win, A_BOLD);
	wmove(win, 6, 6);
	whline(win, '.', 60);
	wmove(win, 9, 6);
	whline(win, '.', 60);	
	wattroff(win, A_BOLD);
	
	// U, B 기본이 아닌 바탕
	wattron(win, COLOR(COLOR_WHITE, COLOR_WHITE));
	
	wmove(win, 6, 6);
	whline(win, ' ', U);
	wmove(win, 9, 6);
	whline(win, ' ', B);
	
	wrefresh(win);
}

