/// @file	dialog.h
/// @brief	dialog.h
#ifndef dialog_h__
#define dialog_h__

#include <ncurses.h>
#include <string>
#include <vector>

///	@brief	MLS namespace
namespace MLS {

enum {
	YN_Y,	///< Yes
	YN_N	///< No
};

int MsgBox(int color, const char *title, const char *msg,...);
int MsgBox(const char *title, const char *msg, int color=COLOR_RED);
WINDOW*  MsgWaitBox(const char *title, const char *msg, int back_color = COLOR_RED);
int MsgWaitEnd(WINDOW*	win);
int InputBox(const char *title, std::string &str, int color=COLOR_MAGENTA);
int YNBox(const char *title,int YN, int back_color=COLOR_CYAN);

int SelectBox(const char *title, std::vector<std::string> &, int n, int back_color=COLOR_MAGENTA);

///	@brief	 상태출력 박스
class ProgressBox
{
protected:
	WINDOW *win;		///< 상태를 출력할 window
	const char *title,	///< title message
		  		*msg;	///<  Message
	int back_color;		///< 배경 색상
		
	std::string sLeftString, sRightString;
	int m_nCounter;  		///< 카운터 60 기준

public:
	ProgressBox(const char *_title, const char *_msg, int back_color=COLOR_MAGENTA);
	~ProgressBox();
	
	void setLeftStr(const char *p)  { sLeftString=p; }
	void setRightStr(const char *p) { sRightString=p; }
	
	void show();
	void redraw();
	
	void setCount(int nCount);
};

///	@brief	 상태출력 박스
class StatBox
{
protected:
	WINDOW *win;		///< 상태를 출력할 window
	const char *title,	///< title message
		  		*msg;	///<  Message
	int back_color;		///< 배경 색상
		
	const char *UL, *UR, *BL, *BR;
	int U, B;  	// 60 기준
				
public:
	StatBox(const char *,const char *, int back_color=COLOR_MAGENTA);
	~StatBox();
	
	void setUL(const char *p){UL=p;}
	void setUR(const char *p){UR=p;}
	void setBL(const char *p){BL=p;}
	void setBR(const char *p){BR=p;}
	
	void setU(int p){U=p; if (U<0) U=0; if (U>60) U=60;}
	void setB(int p){B=p; if (B<0) B=0; if (B>60) B=60;}
			
	void show();
	void redraw();
};

}; // namespace

#endif//dialog_h__
