/// @file	mlslocale.h
/// @brief	locale 설정 관련 소스
/// @author 
/// @date 2004-08-21

#ifndef LOCALE_H__
#define LOCALE_H__

#define ICONV_CONST

#include <iconv.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses.h>
#include <locale.h>
#include <string>
#include <pwd.h>
#include <sys/types.h>
#include <libintl.h>

#include "define.h"
#include "strutil.h"

using namespace std;

namespace MLS
{
	extern chtype HLINE;
	extern chtype VLINE;
	extern chtype ULCORNER;
	extern chtype LLCORNER;
	extern chtype URCORNER;
	extern chtype LRCORNER;
	extern chtype LTEE;
	extern chtype RTEE;
	extern chtype BTEE;
	extern chtype TTEE;
	
	/// @brief	locale language enum
	enum ENCODING {
		KO_EUCKR,	///< korean Language - EUC-KR
		KO_UTF8,	///< korean Language - UTF-8
		US,			///< USA Language
		AUTO		///< AUTO 자동으로 바꿔준다.
	};
	
	/// @brief	Line Code
	enum LINECODE { CHARLINE, ACSLINE };

	extern ENCODING	e_nCurLang; // 현재 시스템의 language 를 세팅한다.
	extern ENCODING	e_nBefLang; 
	
	extern LINECODE e_nBoxLineCode;
	
	void Set_Locale(ENCODING e);
	void Set_Line(MLS::LINECODE nCode);
	
	char* code_convert(	const char *str, 
						const char *to_codeset, 
						const char *from_codeset);
	
	string 		CodeConvert(string sSource, const char *to_codeset,	const char *from_codeset);
	string 		ChgEngKor(string sEnglish, string sKorean);
};

#endif//LOCALE_H__
