#include "commandimp.h"
#include <sys/wait.h>

extern int Copy(MLS::Selection &, std::string);
extern int Move(MLS::Selection &, std::string);

namespace MLS {

void CommandImp::CopyClip()
{
	g_MainFrame->tClipboard.Set(panel->GetSelection(), CLIP_COPY);
}

void CommandImp::CutClip()
{
	g_MainFrame->tClipboard.Set(panel->GetSelection(), CLIP_CUT);
}

void CommandImp::PasteClip()
{
	ClipState state = g_MainFrame->tClipboard.GetState();
	
	if (state == CLIP_NONE) return;
	
	if (state == CLIP_COPY)
	{
		::Copy(g_MainFrame->tClipboard.Get(), panel->path);
		Refresh();
	}
	else if (state == CLIP_CUT)
	{
		::Move(g_MainFrame->tClipboard.Get(), panel->path);
		Refresh();		
	}
}

void CommandImp::ViewScreen( )
{
	def_prog_mode();
	endwin();
	
	cbreak();
	noecho();
		
	getch();
	refresh();
}

///	@brief	명령어를 실행한다.
///	@param	cmd		명령어및 인자
///	@param	bPause	명령어를 실행시키고 종료시 잠tClipboard시 정지시킬지 여부 
/// @param	bFork	Fork할 것인지?
///	@return	명령어 실행 결과
int CommandImp::Run( const std::string &cmd, bool bPause, bool bBackground)
{
	def_prog_mode();
	endwin();
		
	int status;
	
	if (bBackground)
	{
		status = system("clear");
		string sCmd = cmd + " > /dev/null 2>&1 &";
		status = system(sCmd.c_str());
	}
	else
	{
		status = system("clear");
		status = system(cmd.c_str());
	}
	
	if (bPause)
	{
		cbreak();
		noecho();
		
		cout << endl << gettext("Press any key to return Mls...") << endl;
		getch();
	}
	refresh();	
	
	Refresh();
	
	return status;
}

};
