///  @file		main.cpp
///  @brief 	main source file
///  @author	mls programer
///  @date		2004-

#include <stdlib.h>
#include <ncurses.h>

#include <deque>

#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "config.h"

#include "define.h"
#include "mainframe.h"

#include "conf.h"
#include "configure.h"
#include "mlslocale.h"
#include "colorset.h"

using namespace MLS;
using namespace std;

char *logfile ="/tmp/log.txt";	///< log file

namespace {
// 처음 기본 세팅
bool		bMcdExe  = false;	///< MCD실행 여부
ENCODING	nLangSet = AUTO;	///< 언어 설정
LINECODE	nLineSet = ACSLINE;	///< LINE MODE

char *cPrintRED = "31";
char *cPrintBLUE = "36";
char *cPrintYELLOW = "33";

};

namespace MLS {
	
extern deque<string> g_CmdHistory;

bool Initialize();
void Destroy();

/// @brief 	시그널 처리를 하는 함수
/// @param sig	시그널 Number
void signal_action(int sig)
{
	# if (__GNUC__ * 1000 + __GNUC_MINOR__) >= 3000
		LOG("Signal :: [%d] [%s]", sig, strsignal(sig));
	# else
		LOG("Signal :: [%d]", sig);
	# endif
	
	static int nSigCount = 0;
	
	switch(sig)
	{
		case SIGINT:
			// Copy & paste를 위해서 Ctrl-C를 사용합니다.
			ungetch(3);
			return;
	}
	
	g_MainFrame->Destroy();
	delete g_MainFrame;
	
	Destroy();
	
	# if (__GNUC__ * 1000 + __GNUC_MINOR__) >= 3000
		printf("Signal [%d] [%s]\n", sig, strsignal(sig));
	# else
		printf("Signal [%d]\n", sig);
	# endif
	exit(0);
}

/// @brief 시그널블럭 처리를 한다.
/// @return SUCCESS(0)
int signal_blocking()
{
	struct sigaction act;
	
	// 다음은 시그널이 들어왔을 때 실행 함수
	act.sa_handler = signal_action;
	
	// 다음 주석을 해제 하면 시그널 무시
	//act.sa_handler = SIG_IGN;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	
	setpgrp();
	
	sigaction(SIGHUP, &act, 0);		/* 1 Hangup (POSIX).  */
	sigaction(SIGINT, &act, 0);		/* 2 Interrupt (ANSI).  */
	sigaction(SIGQUIT, &act, 0);	/* 3 Quit (POSIX).  */
	sigaction(SIGILL, &act, 0);		/* 4 Illegal instruction (ANSI).  */
	sigaction(SIGTRAP, &act, 0);	/* 5 Trace trap (POSIX).  */
	sigaction(SIGABRT, &act, 0);	/* 6 Abort (ANSI).  */
	sigaction(SIGBUS, &act, 0);		/* 7 BUS error (4.2 BSD).  */
	sigaction(SIGFPE, &act, 0);		/* 8 Floating-point exception (ANSI).  */
//	sigaction(SIGKILL, &act, 0);	/* 9 Kill, unblockable (POSIX).  */
	sigaction(SIGUSR1, &act, 0);	/* 10 User-defined signal 1 (POSIX).  */
//	sigaction(SIGSEGV, &act, 0);	/* 11 Segmentation violation (ANSI).  */
	sigaction(SIGUSR2, &act, 0);	/* 12 User-defined signal 2 (POSIX).  */
	sigaction(SIGPIPE, &act, 0);	/* 13 Broken pipe (POSIX).  */
	sigaction(SIGALRM, &act, 0);	/* 14 Alarm clock (POSIX).  */
	sigaction(SIGTERM, &act, 0);	/* 15 Termination (ANSI).  */
	sigaction(SIGSTKFLT, &act, 0);	/* 16 Stack fault.  */
//	sigaction(SIGCHLD, &act, 0);	/* 17 Child status has changed (POSIX).  */
	sigaction(SIGCONT, &act, 0);	/* 18 Continue (POSIX).  */
//	sigaction(SIGSTOP, &act, 0);	/* 19 Stop, unblockable (POSIX).  */
	sigaction(SIGTSTP, &act, 0);	/* 20 Keyboard stop (POSIX).  */
	sigaction(SIGTTIN, &act, 0);	/* 21 Background read from tty (POSIX).  */
	sigaction(SIGTTOU, &act, 0);	/* 22 Background write to tty (POSIX).  */
	sigaction(SIGURG, &act, 0);		/* 23 Urgent condition on socket (4.2 BSD).  */
	sigaction(SIGXFSZ, &act, 0);	/* 24 CPU limit exceeded (4.2 BSD).  */
	sigaction(SIGXCPU, &act, 0);	/* 25 File size limit exceeded (4.2 BSD).  */
	sigaction(SIGVTALRM, &act, 0);	/* 26 File size limit exceeded (4.2 BSD).  */
	sigaction(SIGPROF, &act, 0);	/* 27 Profiling alarm clock (4.2 BSD).  */
//	sigaction(SIGWINCH, &act, 0);	/* 28 Window size change (4.3 BSD, Sun).  */
	sigaction(SIGIO, &act, 0);		/* 29 I/O now possible (4.2 BSD). Pollable event occurred (System V).  */ 
	sigaction(SIGPWR, &act, 0);		/* 30 Power failure restart (System V).  */
	sigaction(SIGSYS, &act, 0);		/* 31 Bad system call. (Unused) */
	return SUCCESS;
}

/// @brief	mls 시작시 초기화 함수
///
/// 주로 nCurses 초기화를 한다.
///
/// @return	성공여부 0일때 성공
bool Initialize()
{
	printf("Mls %s, user-friendly graphic shell, 2004\n", VERSION);
	
	// 시그널 블럭 처리
	signal_blocking();
	
	Set_Locale(nLangSet); 	// Locale  설정
	
	// configure Initialize
	string 	sViewStr;
	sViewStr = "Initialize Configure";
	
	if (e_tConf.Init() && e_tColor.Init())
	{
		sViewStr.append(60- sViewStr.size(), ' ');
		sViewStr = sViewStr + "[\033[01;" + cPrintBLUE + "m";
		sViewStr = sViewStr + " SUCCESS ";
		sViewStr = sViewStr + "\033[0m]";
		
		cout << sViewStr.c_str() << endl;
	}
	else
	{
		sViewStr.append(60- sViewStr.size(), ' ');
		sViewStr = sViewStr + "[\033[01;" + cPrintRED + "m";
		sViewStr = sViewStr + " FAIL ";
		sViewStr = sViewStr + "\033[0m]";
		
		cout << sViewStr.c_str() << endl;
		return false;
	}
			
	// load Configure	
	const string cfgmain= "/etc/mls/";		       // /etc/mls에서 설정파일을 읽는다.
	string sTmpPath = e_tConf.cfghome + "mls.cfg"; // . mls.cfg 설정 파일을 읽는다.
	
	sViewStr = "Load configuration " + sTmpPath;
	
	if (e_tConf.load(sTmpPath.c_str()) == ERROR)
	{
		sViewStr.append(60- sViewStr.size(), ' ');
		sViewStr = sViewStr + "[\033[01;" + cPrintRED + "m";
		sViewStr = sViewStr + " FAIL ";
		sViewStr = sViewStr + "\033[0m]";
		
		cout << sViewStr.c_str() << endl;
				
		sTmpPath = cfgmain + "mls.cfg";
		
		sViewStr = "Load configuration " + sTmpPath;
		if (e_tConf.load(sTmpPath.c_str()) == ERROR)
		{
			sViewStr.append(60- sViewStr.size(), ' ');
			sViewStr = sViewStr + "[\033[01;" + cPrintRED + "m";
			sViewStr = sViewStr + " FAIL ";
			sViewStr = sViewStr + "\033[0m]";
			return false;
		}
	}
	
	sViewStr.append(60-sViewStr.size(), ' ');
	sViewStr = sViewStr + "[\033[01;" + cPrintBLUE + "m";
	sViewStr = sViewStr + " SUCCESS ";
	sViewStr = sViewStr + "\033[0m]";
	
	cout << sViewStr.c_str() << endl;
	
	// . colorset을 load한다.
	sTmpPath = e_tConf.cfghome + e_tConf.colorset;
	
	sViewStr = "Load colorset  " + sTmpPath;
	if (e_tColor.Load(sTmpPath.c_str()) == false)
	{
		sViewStr.append(60- sViewStr.size(), ' ');
		sViewStr = sViewStr + "[\033[01;" + cPrintRED + "m";
		sViewStr = sViewStr + " FAIL ";
		sViewStr = sViewStr + "\033[0m]";
		
		cout << sViewStr.c_str() << endl;
		
		sTmpPath = cfgmain + e_tConf.colorset;
		
		sViewStr = "Load colorset  " + sTmpPath;
		if (!e_tColor.Load(sTmpPath.c_str()))
		{
			sViewStr.append(60- sViewStr.size(), ' ');
			sViewStr = sViewStr + "[\033[01;" + cPrintRED + "m";
			sViewStr = sViewStr + " FAIL ";
			sViewStr = sViewStr + "\033[0m]";
			
			cout << sViewStr.c_str() << endl;
			return false;
		}
	}
	
	sViewStr.append(60- sViewStr.size(), ' ');
	sViewStr = sViewStr + "[\033[01;" + cPrintBLUE + "m";
	sViewStr = sViewStr + " SUCCESS ";
	sViewStr = sViewStr + "\033[0m]";
	
	cout << sViewStr.c_str() << endl;
	
	// history를 읽어온다 없어도 상관없다
	ifstream in((e_tConf.cfghome + "history").c_str());
	if (in)
	{
		string line;
		while(!getline(in, line).eof())
		{
			if (line.empty()) continue;
			g_CmdHistory.push_back(line);
		}		
	}	
	
	return true;
}

/// @brief	mls가 종료될때 호출되는 함수
void Destroy()
{
	// history를 저장한다 없어도 상관없다	
	ofstream out((e_tConf.cfghome + "history").c_str());		
	if (out)
		for (int t = 0; t<g_CmdHistory.size(); t++)
			out << g_CmdHistory[t] << endl;
	
}

/// @brief	print help
void PrintHelp(void)
{
	string sStr_Ko, sStr_En, sConvert;
	
	sStr_Ko = 	"도스용 파일관리 툴인 Mdir의 리눅스 클론을 입니다.\n"
  				"기존의 Mdir의 기본 단축키와 기본기능을 모방한 프로그램입니다.\n"  
  				"프로그램의 기능 버그, 추가될 사항이 있으시면 프로젝트\n"
  				"홈페이지나, 이메일을 통해서 연락주시면 감사하겠습니다.\n\n"
				" * 프로젝트 홈페이지 : http://kldp.net/projects/mls/\n";
	sStr_Ko += " * Option\n";
	sStr_Ko += "\t --help     : 도움말\n";
	sStr_Ko += "\t --en       : 영문 실행\n";
	sStr_Ko += "\t --ko       : 한글 버전(utf8)\n";
	sStr_Ko += "\t --ko_euckr : 한글 버전(euc_kr)\n";
	sStr_Ko += "\t --noline   : 선형태를 -,|,+ 로 바꿈\n";
	sStr_Ko += "\t --mcd      : 바로 MCD 실행 \n";
	
	sStr_En = 	"Mls is a visaul file manager.\n"
		"It's a feature rich full-screen text mode application that allows you to copy, move and\n"
		"delete files and whole directory trees, search for files and run commands in the subshell.\n\n"
		" * project homepage : http://kldp.net/projects/mls/\n";
		
	sStr_En += " * Option\n";
	sStr_En += "\t --help     : this page\n";
	sStr_En += "\t --en       : english version\n";
	sStr_En += "\t --ko       : korean version (utf8)\n";
	sStr_En += "\t --ko_euckr : korean version (euc_kr)\n";
	sStr_En += "\t --noline   : linetype change (-,|,+)\n";
	sStr_En += "\t --mcd      : mcd excute\n";	
	
	cout << ChgEngKor(sStr_En, sStr_Ko).c_str() << endl;
}

/// @brief	Option 처리함수
/// @return true : 지원하는 옵션 \n
///			false : 지원하지 않는 옵션
bool OptionProc(int argc, char *argv[])
{
	int         nCount;
	string		sArg;
		
	if (argc >= 2)
	{
		for (nCount=1;nCount < argc;nCount++)
		{
			sArg = argv[nCount];
			if (sArg == "--help")
			{
				Set_Locale(AUTO);
				PrintHelp();
				return false;
			}
			else if (sArg == "--noline")
			{
				e_nBoxLineCode = CHARLINE;
			}
			else if (sArg == "--en")
			{
				nLangSet = US;
			}
			else if (sArg == "--ko")
			{
				nLangSet = KO_UTF8;
			}
			else if (sArg == "--ko_euckr")
			{
				nLangSet = KO_EUCKR;
			}
			else if (sArg == "--mcd")
			{
				bMcdExe = true;
			}
			else
			{
				Set_Locale(AUTO);
				PrintHelp();
				return false;
			}
		}
	}
	return true;
}

};

/// @brief	진입부
/// @param	argc	인자 개수
/// @param	argv	인자
/// @return	성공여부
int main(int argc, char *argv[])
{
	if (!OptionProc(argc, argv)) return ERROR;
	if (!Initialize())	return ERROR; // . 시스템 초기화
	
	g_MainFrame = new MainFrame;
	
	if (g_MainFrame->Init())
	{
		g_MainFrame->Do(bMcdExe);
		g_MainFrame->Destroy();
	}
	
	delete g_MainFrame;
	Destroy();
	
	return SUCCESS;
}
