/**
 * @file	mainframe.cpp
 * @brief	mainframe source file 
 * @author	la9527
 * @date	2004-08-22
 */
#include <sys/types.h>
#include <signal.h>

#include "define.h"
#include "mainframe.h"
#include "mlslocale.h"
#include "dialog.h"
#include "configure.h"

namespace MLS {

MainFrame *g_MainFrame;

MainFrame::MainFrame()
{
	bSplit  = false;
	nActive = 0;
	
	// Thread 
	//m_pMlsMutex = new MlsMutex;
	//m_pGetKey = new GetKey(m_pMlsMutex);
}

MainFrame::~MainFrame()
{
	//delete m_pGetKey;
	//delete m_pMlsMutex;
}

bool MainFrame::Init()
{
	initscr();     // nCurses 시작
	start_color(); // 컬러 가능하게 
	
	nonl();        // . 엔터 처리?
	raw();		   // Ctrl+C 처리 가능하게 
	//cbreak();      // . nCurses : 즉시 입력이 가능하도록 설정
	noecho();	   // . nCurses : echo모드 해제
	keypad(stdscr, TRUE);  // . keypad 활성화
	
	curs_set(0); // 커서를 보이지 않게 한다.

	// . 색깔 쌍을 초기화 한다. 0 ~ 49개 까지
	for (int t=0; t<8; t++)
		for (int t2=0; t2<8; t2++)
	{
		if (t==0 && t2==0) continue;
		init_pair(t*8+t2, t, t2);
	}
		
	// curses 마우스 설정		
	mousemask( BUTTON1_CLICKED | BUTTON2_CLICKED | BUTTON_SHIFT | BUTTON_CTRL, NULL);
	mouseinterval(30);
	
	// line 설정
	Set_Line(e_nBoxLineCode);
	
	bShell = false;
	
	// GetKey Thread Start
	//m_pGetKey->Start(NULL);
	return true;
}

void MainFrame::Destroy()
{
	clear();
	refresh();
	
	// . keypad 비활성화
	keypad(stdscr, FALSE);  // . 라인단위로 입력이 이루어지도록 설정	
	//nocbreak();
	noraw();
	curs_set(1); // 커서를 다시 보이게 한다.
	noecho();
	endwin();   // . nCurses 해제
}

void MainFrame::SetSplitWindow( )
{
	if (bSplit)
	{
		tPanel[0].SetWindow(win1, (COLS-2)/2, LINES-6);
		tPanel[1].SetWindow(win2, (COLS-1)/2, LINES-6);		
	}
	else
	{
		tPanel[nActive].SetWindow(win0, COLS, LINES-6);	
	}
}

void MainFrame::Draw()
{
	int i;
	// . Function Panel Draw
	setcol(e_tColor.Func);
	move(0,0);
	hline(' ', COLS);
	
	int f = COLS / 8, pos, bound = 7-COLS % 8;
	
	if (!bound) f++;
		
	for (i=0; i<8; i++)
	{
		if (!bound || i < bound) pos = i*f;
		else pos = i*(f+1) - bound + 1;
		
		mvprintw(0, 2+pos,"%s", e_tConf.Func[i+1].getName());
	}
	unsetcol();
	
	setcol(e_tColor.FuncA);
	mvprintw (0, 0, "F");
	for (i=0; i<8; i++)
	{
		if (!bound || i < bound) pos = i*f;
		else pos = i*(f+1) - bound + 1;		
		mvprintw(0, 1+pos, "%d", i+2);
	}
	unsetcol();	
	
	if (e_tColor.Func.back >=8 ) attron(A_BLINK);
	attron(COLOR(COLOR_BLACK, e_tColor.Func.back%8));
	for (i=1; i<8;i++)
	{
		if (!bound || i < bound) pos = i*f;
		else pos = i*(f+1) - bound + 1;
		mvaddch(0, pos, VLINE);
	}
	unsetcol();

	// . PATH Draw
	setcol(e_tColor.Stat);
	move(1, 0);
	hline(' ', COLS);
	if (bSplit)
	{
		printw("%s", tPanel[0].GetPathView().c_str());
		move(1, COLS/2+1);
		printw("%s", tPanel[1].GetPathView().c_str());
	}
	else
	{	
		printw("%s", tPanel[nActive].GetPathView().c_str());
	}
	unsetcol();
	
	// 위 아래 선 및 경계선 긋기..
	setcol(e_tColor.Line);	
	move(2,0);	hline(HLINE, COLS);	
	move(LINES-3,0);	hline(HLINE, COLS);
	
	// 2단일 경우.
	if (bSplit)
	{
		for (int t=0; t<LINES-6; t++)
		{
			mvaddch(3+t, (COLS-2)/2, VLINE);	
			addch(VLINE);
		}
		mvaddch(2, (COLS-2)/2, TTEE);
		addch(TTEE);
		mvaddch(LINES-3, (COLS-2)/2, BTEE);
		addch(BTEE);
	}
	
	//
	unsetcol();
	
	if (bSplit)
	{
		for (int t=0; t<2; t++)
		{
			tPanel[t].draw(nActive== t);
		}
	}
	else
	{
		tPanel[nActive].draw(true);
	}
	
	tPanel[nActive].DrawStatus();
	refresh();
}

void MainFrame::Refresh()
{
	wresize(win0, LINES-6, COLS);
	wresize(win1, LINES-6, (COLS-2)/2);
	wresize(win2, LINES-6, (COLS-1)/2);
	
	mvwin(win1, 3, 0);
	mvwin(win2, 3, (COLS+2)/2);
	
	Draw();
	SetSplitWindow();
	refresh();
}

///	@brief 메인프레임 수행
void MainFrame::Do(bool bMcd)
{

	win0 = newwin(LINES-6, COLS, 3, 0);
	win1 = newwin(LINES-6, (COLS-2)/2, 3, 0);
	win2 = newwin(LINES-6, (COLS-1)/2, 3, (COLS+2)/2);
	
	bExit = false;
	
	// 메뉴
	tMenu.Create();
	
	// 자신의 디렉토리에서 먼저 읽고 없으면 /etc에서 읽는다.
	if (tKeybind.Load(e_tConf.cfghome + "mls.key") == ERROR)
	{
		if (tKeybind.Load("/etc/mls/mls.key") == ERROR)
		{
			SetSplitWindow();	
			refresh();
			MsgBox(	gettext("Error"), gettext(	"Mls config file not found.."));
			kill(getpid(), 3);
		}
	}
	
	tCommand.Init();
		
	tPanel[0].Read(".");
	tPanel[1].Read(".");
	tCommand.SetPanel(&tPanel[0]);
	
	SetSplitWindow();
	
	refresh();
	
	if (bMcd)
		tCommand.Execute((e_tConf.Func[9])[0]);
	
	// ESC 딜레이 줄이기.
	ESCDELAY = 10;
	
	// 에러 번호 체크
	int nErrno = 0;
	
	while(!bExit)
	{
		// 패널 화면을 그린다.
		Draw();
		
		// 쉘을 실행하면 계속 실행하게 한다.
		if (bShell == true) 
		{	
			tCommand.Execute("<shell>");
			continue;
		}
		
		// 키를 입력 받는다.
		int nKey = getch();
		
		LOG("KEY NUM :: %d", nKey);
		
		// 쓰레드를 이용해 키 수신 
		/*
		int nKey = (int)m_pMlsMutex->Wait();
		m_pMlsMutex->WaitEnd();
		*/
		
		// 콘솔에 따라서 백그라운드로 남아 있는 것을 잡기 위한 방법
		if (nKey == -1)	{
			nErrno++;
			if (nErrno == 10) kill(getpid(), 3); // Signal Quit로 종료
			continue;
		} else {
			nErrno = 0;
		}
		
		if (nKey == 3)
		{
			nKey = getch();
			tCommand.Execute("<copyclip>");
			continue;
		}
		
		// 리사이즈 		
		if (nKey == KEY_RESIZE)
		{
			Refresh();
			continue;
		}
		
		// 마우스		
		if (nKey == KEY_MOUSE)
		{
			MEVENT event;
			getmouse(&event);
			
			//if (event.bstate & BUTTON1_CLICKED)
			//	MsgBox("Notify", "Button1 Pressed");
		}	
						
		// Searching..
		if (tPanel[nActive].SearchProcess(nKey)) continue;
		
		// Function key
		if (KEY_F(1) <= nKey && nKey <=KEY_F(12))
		{
			if (e_tConf.Func[nKey-KEY_F(1)].size()>=1)
				tCommand.Execute((e_tConf.Func[nKey-KEY_F(1)])[0]);
			continue;
		}
		
		// Execute 
		if (nKey != 27)
			tCommand.Execute(tKeybind.GetCommand(nKey));
		else
		{
			// Alt + 문자는 이렇게 받는다. 
			nodelay(stdscr, TRUE);
			nKey = getch();
			nodelay(stdscr, FALSE);
			
			if (nKey != ERR)
			{
				tCommand.Execute(tKeybind.GetMetaCommand(nKey));
			}
			else
			{
				tCommand.Execute(tKeybind.GetCommand(27));
			}
		}
	}
}

/*!
    \fn MLS::MainFrame::NextWindow()
 */
void MainFrame::NextWindow()
{
    /// @todo implement me
	if (bSplit)
	{
		nActive = (nActive + 1) %2;
		tCommand.SetPanel(&tPanel[nActive]);
		
		// 디렉토리 이동
		chdir(tPanel[nActive].GetPathView().c_str());
	}
}

};  // namespace

void MLS::MainFrame::RefreshPanel( )
{
	if (bSplit)
	{
		tPanel[0].Refresh();
		tPanel[1].Refresh();		
	}
	else
	{
		tPanel[nActive].Refresh();
	}		
}

