/**
 * @file	mainframe.h
 * @brief	mainframe header file
 * @author	la9527
 * @date	2004-08-22
 */
#ifndef MLSMAINFRAME_H
#define MLSMAINFRAME_H

#include "define.h"
#include "panel.h"
#include "DirMcd.h"
#include "command.h"
#include "clipboard.h"
#include "colorset.h"
#include "getkey.h"

namespace MLS {
/**
	@brief	MainFrame 클래스

	Panel들을 관리한다.
*/
					
class MainFrame
{
private:
	Panel   	tPanel[2];
	Command     tCommand;
	
	Menu    	tMenu;
	Config  	tConfig;
	ClipBoard   tClipboard;
	
	GetKey*		m_pGetKey;
	
	bool        bExit,
				bShell,
				bSplit;   /// < 창나누기인가 아닌가.
	int			nActive;  /// < Active한 창을 가리킨다 0, 1
	
	WINDOW 		*win0, *win1, *win2;

public:
	MlsMutex*	m_pMlsMutex;
	KeyBind 	tKeybind;
		
	MainFrame();
	~MainFrame();
	
	bool    Init();
	void    Destroy();
	
	void 	Do(bool bMcd = false);
	void    Draw();
	void	SetSplitWindow();
	void    Exit() { bExit = true; }
	void	ShellExe(bool Shell) { bShell = Shell; }
    void	NextWindow();
	void	Refresh();
	void    RefreshPanel();
	void	ReloadMenu()
	{ 
		tMenu.Create();
	}
	
	friend class CommandImp;
	friend class Menu;
	friend class Panel;
};

extern MainFrame *g_MainFrame;
};

#endif
