///	@file	panel.cpp
///	@brief	panel.cpp
#include <ncurses.h>
#include <map>
#include <sys/stat.h>

#include "mls.h"
#include "panel.h"
#include "mainframe.h"

using namespace strutil;
using namespace MLS;

/// @brief	생성자
Panel::Panel()
{
	cur=0;
	page=0;
	
	sel_size=0;
	sel_num=0;
	
	// Zip 정의
	m_nZip = 0;
	m_sZipFile = "";
	m_zipFullPath = "/";
	
	bExit=false;	
	bSearch = false;
	
	lines = LINES;
	cols = COLS;
}

/// @brief	소멸자
Panel::~Panel()
{
	delwin(win);
}

/// @brief	비슷한 이름을 가진 파일을 찾는다.
///	@param	str		찾을 파일 앞자리나 전체 파일명
/// @param  s_index 파일을 찾을 첫번째 인덱스
///	@return	찾은 파일 index, 실패시 -1
int 
Panel::Search(const string &str, int s_index)
{
	if (file_list.empty()) return -1;
	
	int l = file_list.size(), len = str.size();
	string p=tolower(str);
	
	//if (l < s_index) s_index = s_index % l;
	
	for (int t=0; t < l ; t++)
	{
		int x = (t + s_index) % l;
		
		if ( file_list[x]->name.size() < str.size() ) continue;
		if ( tolower(file_list[x]->name.substr(0, len)) == p)
		{
			return x;
	 	}
	}
	// 없으면
	return -1;	
}


/// @brief 엔터키
void
Panel::Key_Enter()
{
	int key, bkey;
	bool bSearch = false;
	
	string tTmp;
	string::size_type	tSizeType;
	
	if (m_nZip == 0)
	{
		LOG("Read :: %s", file_list[cur]->name.c_str());
		if (Read(file_list[cur]->name))
		{
			LOG("Read Success");
			return;
		}
	}
	else
	{
		if (file_list[cur]->FullName == "/")
		{
			// new된 Zip 을 삭제
			m_tReader->Destroy();
			m_nZip = 0;
			
			Read(GetCurrentPath());
			return;
		}
		
		if (file_list[cur]->bDir)
		{
			// 압축 파일은 FullName 으로 한다.
			LOG("Read Zip :: %s", file_list[cur]->FullName.c_str());
			if (file_list[cur]->name == "..") // .. 구현
			{
				tTmp = file_list[cur]->FullName;
				tTmp = tTmp.substr(0, tTmp.size() - 1);
				
				tSizeType = tTmp.rfind("/");
				tTmp = tTmp.substr(0, tSizeType+1);
				
				m_zipPath = m_sZipFile + " :: /" + tTmp;
				m_zipFullPath = "/" + tTmp;
				if (Read(tTmp)) return;
			}
			else
			{
				m_zipPath = m_sZipFile + " :: /" + file_list[cur]->FullName;
				m_zipFullPath = "/" + file_list[cur]->FullName;
				if (Read(file_list[cur]->FullName)) return;
			}
			return;
		}
	}
		
	if (!file_list[cur]->bDir)
	{
		WINDOW*	pWin = NULL;
		
		if (m_nZip == 0)
		{
			// 파일 실행	
			std::map<std::string, Action>::iterator i	= e_tConf.ext_prog.find( tolower( file_list[cur]->ext()) );
			
			if (i != e_tConf.ext_prog.end())
			{
				if ((*i).second.size()==1)	
				{
					// 확장자가 압축화일이면 내용을 보여준다.
					if (((*i).second)[0] == "<zip>")
					{
						try
						{
							m_nZip = 1;
							m_sZipFile = file_list[cur]->name;
							m_zipPath = m_sZipFile + " :: /";
							if (Read("") == true) 
								return;
							else 
								m_nZip = 0;
							
						}
						catch(Exception& ex)
						{
							MsgBox(1, gettext("Error"), "%s", (char*)ex);
						}
					}
					else
					{
						g_MainFrame->tCommand.Execute(((*i).second)[0]);
					}
				}
			}
			else if (file_list[cur]->attr[3] == 'x')
			{
				std::string cmd;
				cmd = "./" + file_list[cur]->name;
				
				g_MainFrame->tCommand.Execute(cmd);
			}
		}
		else
		{
			string	sTmp;
			File*	pFile	= file_list[cur];
			
			sTmp = pFile->name;
			
			try
			{
				// 압축 파일 /tmp에 풀고 filename 리턴
				LOG("DataName :: [%s]", sTmp.c_str());
				
				// 파일 실행
				std::map<std::string, Action>::iterator i = e_tConf.ext_prog.find( tolower( pFile->ext()) );
				
				if (i != e_tConf.ext_prog.end())
				{
					// 파일 실행
					if ((*i).second.size()==1)
					{
						g_MainFrame->tCommand.Execute(((*i).second)[0]);
					}
				}
				else
				{
					MsgBox(	gettext("ERROR"), gettext("Not execute file. "));
				}
			}
			catch(Exception& ex)
			{
				MsgBox(1, gettext("Error"), "%s", (char*)ex);
			}
			return;
		}
	}
}

/// @brief RPM, Zip Install
void
Panel::Key_ZipInstall()
{
	if (m_nZip == 0)
	{
		WINDOW*	pWin = NULL;
		string 			sTargetDir;
		
		TargzUtil	tTargzUtil(path, file_list[cur]->name, 0);
		
		if (tTargzUtil.GetFileInfo() == ERROR)
		{
			LOG("ERROR :: GetFileInfo()");
		}
		
		ZIPTYPE 		nZipType = tTargzUtil.GetZipType();
		
		LOG("FILENAME [%s] [%d]", file_list[cur]->name.c_str(), tTargzUtil.GetZipType());
		
		switch(nZipType)
		{
			case TAR_GZ:
			case TAR_BZ:
			case ZIP:
			case TAR:
			{
				try
				{
					//pWin = MsgWaitBox(	gettext("Wait"), gettext(	"Please wait !!! - Cancel Key [ESC]"));
					MLS::DirMcd mcd;
					
					Progress 	tProgress(gettext("Wait"), gettext(	"Please wait !!! - Cancel Key [ESC]"));
					tProgress.Start((Counter*)&mcd, JOIN);
					mcd.SetCounter(0);
					mcd.AddDirectory(GetCurrentPath());
					mcd.setCur(GetCurrentPath());
					mcd.SetCounter(100);
					tProgress.SetRun(PTHREAD_END);
					tProgress.End();
					if (mcd.proc())
						sTargetDir = mcd.getCur();
				}
				catch(Exception& ex)
				{
					MsgBox(1, gettext("Error"), "%s", (char*)ex);
				}
				
				if (sTargetDir != "") 
				{
					try
					{
						string sView = gettext("Please wait !!! - Uncompress file [") + file_list[cur]->name + "]";
						
						Progress 	tProgress(gettext("Wait"), sView.c_str());
						tProgress.Start((Counter*)&tTargzUtil, JOIN);
						m_tReader->SetCounter(0);
						tTargzUtil.Uncompress(sTargetDir);
						m_tReader->SetCounter(100);
						tProgress.SetRun(PTHREAD_END);
						tProgress.End();
					}
					catch(Exception& ex)
					{
						MsgBox(1, gettext("Error"), "%s", (char*)ex);
					}
					
					// 압축을 풀면 풀린디렉토리로 이동
					Read(sTargetDir);
				}
				break;
			}
				
			case RPM:
			{
				vector<string> q;
				q.push_back(gettext("Install"));
				q.push_back(gettext("Install Nodeps"));
				q.push_back(gettext("Cancel"));
				
				int nSelect = SelectBox(file_list[cur]->name.c_str(), q, 0);
				
				string sCommand;
				
				switch(nSelect)
				{
					case 0:
						sCommand = "rpm -Uvh " + path + file_list[cur]->name;
						break;
						
					case 1:
						sCommand = "rpm -Uvh --nodeps " + path + file_list[cur]->name;
						break;
						
					case 2:
						return;
						break;
				}
				
				LOG("RPM Install [%s]", sCommand.c_str());
				g_MainFrame->tCommand.Run(sCommand, true);
				break;
			}
			
			default:
			{
				MsgBox(	gettext("Error"), gettext(	"Current file not install."));
			}
		}
	}
	else
	{
		MsgBox(	gettext("Error"), gettext(	"zip file not install.."));
	}
}

/// @brief	왼쪽 방향키
void
Panel::Key_Left()
{
	if (cur < (unsigned)nrow) 
		cur=0;
	else 
		cur-=nrow;
}

/// @brief	오른쪽 방향 키
void
Panel::Key_Right()
{
	cur+=nrow;
	if (cur > file_list.size()-1) cur=file_list.size()-1;
}

/// @brief	위쪽 방향키
void
Panel::Key_Up()
{
	if (cur > 0) cur--;
}

/// @brief	아래쪽 방향키
void
Panel::Key_Down()
{
	if (cur < file_list.size()-1) cur++;
}

/// @brief	page up key
void
Panel::Key_PageUp()
{
	if (cur < (unsigned)nrow*ncol) cur=0;
			else cur-=(nrow*ncol);
}

/// @brief	page down key
void
Panel::Key_PageDown()
{
	cur+=(nrow*ncol);
	if (cur > file_list.size()-1) cur=file_list.size()-1;
}

/// @brief	home key
void
Panel::Key_Home()
{
	cur = 0;
}

/// @brief	Key End
void
Panel::Key_End()
{
	cur = file_list.size()-1;
}

/// @brief  키를 입력받아 파일 찾기
/// @param 
/// @return 찾았는가 못찾았는가
bool
Panel::SearchProcess(int key)
{
	int tmp;
	
	if (!bSearch && ( isalnum(key) || key == '.' ) ) bSearch = true;
	
	if (bSearch && key == '\t')
	{// 탭이 나오면 다음 것을 찾는다.
		if ((tmp = Search(strSearch, cur+1)) != -1)
			cur = tmp;
		return true;
	}
		// .. 파일을 찾아 커서를 파일위에 둔다
	if (bSearch &&  key != '/' && ( (32<= key && key <=126) || key == KEY_BS ) )
	{
		// search mode;
		// 뒷부분에 key를 넣고
		
		if (key == KEY_BS) 
		{
			strSearch.erase(strSearch.size()-1, 1);
			if (strSearch.empty()) {bSearch = false; return true;}
		}
		else strSearch += (char) key;
		
		// strSearch로 시작하는 문자를 찾는다..						
		if ((tmp = Search(strSearch, cur))== -1)		
		{			
			// 없으면 처음에 넣고 다시 찾는다.
			if ((tmp = Search(string(1, key)))!= -1)
			{
				strSearch = (char)key;
				cur = tmp;
			}
			else strSearch.erase(strSearch.size()-1, 1);
		}
		else cur = tmp;
		return true;
	}
	
	bSearch = false;
	strSearch.erase();
	
	return false;
}

/// @brief	파일선택
void
Panel::ToggleSelect()
{
	if (file_list[cur]->selected) deselect(*file_list[cur]);
	else select(*file_list[cur]);
			
	if (cur < file_list.size()-1) cur++;
}

void 
Panel::SetWindow( WINDOW *_win, int w, int h)
{
	win = _win;
			
	lines=h;
	cols=w;
}
