/// @file	panel.h
///	@brief	panel	headerfile
#ifndef MLSmain_H__
#define MLSmain_H__

#include <list>
#include <vector>
#include <string>

#include "define.h"
#include "file.h"
#include "drawutil.h"
#include "strutil.h"
#include "mlslocale.h"
#include "keydef.h"
#include "dialog.h"
#include "conf.h"

#include "DirMcd.h"
#include "TargzUtil.h"

#include "mpool.h"
#include "reader.h"
#include "history.h"
#include "menu.h"
#include "keybind.h"
#include "colorset.h"
#include "selection.h"
#include "progress.h"

typedef unsigned int uint;		///< unsigned int 재정의

namespace MLS {

///	@brief	Panel class
class Panel
{
// Draw Section ( 플랫폼에 따라 어떻게 바뀔지 모르겠음 ) 
protected:
	WINDOW *win;					///< 출력할 윈도우
	int cols, lines;
	
// 일반 처리
protected:	
	typedef vector<File *>::iterator pFileIterator;///< vector<File *>
														/// 재정의	
	MemoryPool<File> pool;			///< pool
	vector<File *> file_list;   ///< 지금 리스트에 나타낼 파일들.,	
	
	History tHistory;				///< history	
		
	string path;				///< path
	string strSearch;			///< 찾고 있는 파일 문자
	
	uint sel_size;					///< 선택한 파일들 크기
	uint sel_num;					///< 선택된 파일들의 개수
			
	uint cur,						///< 커서 위치(file index값)
		 page;						///< page	
	int  ncol,						///< ncol
		 nrow,						///< 파일정보를 출력할 줄 개수
		 npage;						///< 1페이지 당 줄 개수
	uint ndir,						///< ndir
		 nfile,						///< nfile
		 dir_size;					///< dir_size
	
	bool bExit;						///< 프로그램을 끝내는 여부
	bool bRoot;						///< root direct 여부
	bool bSearch;					/// 서치모드인지 여부
	
	// Zip파일을 위한
	int		m_nZip;					///< nZip
	string	m_zipPath;				///< zipPath
	string	m_zipFullPath;			///< zipFullPath
	string	m_sTmpPath;				///< TmpPath
	string	m_sZipFile;				///< sZipFile
	
	Reader *m_tReader;				///< tReader	
	
public:
	Panel();
	~Panel();	
	void proc();
	
	// Windows
	void SetWindow(WINDOW *win, int w, int h);
	
	// commands	
	void Refresh();
	
	int about();
	int cmd_help();
	int cmd_rename();
	int cmd_view();
	int cmd_edit();
	int cmd_copy();
	
	int cmd_zip_copy(void);
	int cmd_zip_remove(void);
	int cmd_move();
	int cmd_remove();
	int cmd_menu();
	int cmd_attr();
	int cmd_mcd();
	
	// read	
	bool Read(const string &);
	
	void         Init();
	void         Sort();
	Reader		*GetReader(const string &path);	
	ColorEntry  *GetColor(File &ext);
	
	string		ViewCurrentPath(void);
	string		GetRealPath(const string &);
	string		GetCurrentPath(void);
	File		GetCurrentFile(void);
	
	void		SetZipCurrentPath(string sCurPath)  
	{ 
		if (sCurPath.substr(0, 1) == "/")
			m_zipFullPath = sCurPath;
		else
			m_zipFullPath = "/" + sCurPath;
	}
	void		SetPathView(string sCurPath)
	{ 
		if (sCurPath.substr(0, 1) == "/")
			m_zipPath = m_sZipFile + " :: " + sCurPath; 
		else
			m_zipPath = m_sZipFile + " :: /" + sCurPath; 
	}
	
	string		GetZipCurrentPath(void)
	{
		LOG("GetZipCurrentPath [%s] --------", m_zipFullPath.c_str());
		return m_zipFullPath;
	}
	
	string		GetPathView(void)
	{
		if (isZip() == true)
			return m_zipPath;
		else
			return path;
	}
	
	vector<MLS::File*> GetZipFiles()
	{ 
		vector<File*>	vFiles;
		
		if (isZip() == true)
			vFiles = m_tReader->GetFiles();
		return vFiles;
	}
	
	// draw	
	void draw(bool bDrawCursor = false);
	void drawfile1(bool);
	void drawfile2(bool);
	void drawfile3(bool);
	void drawfile4(bool);
	void drawfile5(bool);
	void drawfile6(bool);
	
	void drawmode1(int, bool);
	void drawmode2(int, bool);
	void drawmode3(int, bool);
	void drawmode4(int, bool);
	
	void DrawStatus();
	
	// select
	void select(File &);
	void deselect(File &);	
	
	Selection GetSelection();
	
	// search
	int  Search(const string &, int start = 0); 
	bool SearchProcess(int);
    
	bool isZip()
	{
	 	if (m_nZip != 0) 
			return true;
		else
			return false;
	}
	
	friend class CommandImp;
	
	void	Key_ZipInstall();
	
	// 방향키 설정
	void	Key_Enter();
	void	Key_Left();
	void	Key_Right();
	void	Key_Up();
	void	Key_Down();
	void	Key_PageUp();
	void	Key_PageDown();
	void	Key_Home();
	void	Key_End();
	
	void    ToggleSelect();
	
protected:

/// @brief	directory인지 확인을 하는 class
class is_dir
{
public:
	/// @brief	operator () 정의
	///
	/// directory인지 확인하는 함수
	///
	/// @param	*a	directory인지 확인할 파일
	/// @return	true/false
	bool operator()(File *a)
	{
		return a->bDir;
	}
};

///	@brief	file color를 비교하는 class
class sort_color
{
public:
	/// @brief	operator () 정의
	///
	/// file color로 sort시 필요한 함수로써 두개의 file color을 비교한다
	///
	/// @param	*a	첫번째 파일
	/// @param	*b	두번째 파일
	/// @return	true/false
	bool operator()(File *a, File *b)
	{		
		if (a->color==b->color)	return a->name < b->name;
		
		if (*a->color == e_tColor.Default) return false;
		if (*b->color == e_tColor.Default) return true;
		
		return *a->color < *b->color;
	}

};

/// @brief	file name으로 비교하는 class
class sort_name
{
public:

	/// @brief	operator () 정의
	///
	/// filename으로 sort시 필요한 함수로써 두개의 filename을 비교한다
	///
	/// @param	*a	첫번째 파일
	/// @param	*b	두번째 파일
	/// @return	true/false
	bool operator()(File *a, File *b)
	{
		return a->name < b->name;		
	}
};


///	@brief	파일 확장자로 파일을 비교하는 class
class sort_ext
{
public:
	/// @brief	operator () 정의
	///
	/// 확장자 sort시 필요한 함수로써 두개의 파일확장자를 비교한다
	///
	/// @param	*a	첫번째 파일
	/// @param	*b	두번째 파일
	/// @return	true/false
	bool operator()(File *a, File *b)
	{
		if (a->ext()==b->ext())	return a->name < b->name;
		return a->ext() < b->ext();
	}
};

///	@brief	file size로 비교하는  class
class sort_size
{
public:
	/// @brief	operator () 정의
	///
	/// file size로 sort시 필요한 함수로써 두개의 file size를 비교한다
	/// @param	*a	첫번째 파일
	/// @param	*b	두번째 파일
	/// @return	true/false
	bool operator()(File *a, File *b)
	{
		if (a->size==b->size)	return a->name < b->name;
		return a->size < b->size;	
	}
};

///	@brief	파일을 최근 수정 시간을 비교하는 class
class sort_time
{
public:
	/// @brief	operator () 정의
	///
	/// file mtime(time of last modification)으로 sort시 필요한 함수로써 두개의 
	/// file mtime를 비교한다
	/// @param	*a	첫번째 파일
	/// @param	*b	두번째 파일
	/// @return	true/false
	bool operator()(File *a, File *b)
	{
		if (a->mtime==b->mtime)	return a->name < b->name;
		return a->mtime < b->mtime;
	}
};

};

};

#endif
