///	\file	panel_draw.cpp
///	\brief	panel_draw.cpp
#include <curses.h>
#include <vector>
#include <algorithm>

#include "mls.h"
#include "panel.h"
#include "conf.h"
#include "mlslocale.h"
#include "drawutil.h"
#include "strutil.h"
#include "dialog.h"

using namespace std;
using namespace strutil;
using namespace MLS;

char kindsort[6] = {' ', 'N', 'E', 'S', 'T', 'B'};	///< 파일 정렬 방식

class StringLengthCompare{
public:
	bool operator() (File *a, File *b)
	{
		return (a->name.size() < b->name.size());
	}
};
		

///	\brief	화면에 mls를 그린다.
void Panel::draw(bool bDrawCursor)
{
	int i;
	
	setcol(e_tColor.Default, win);	
	for (int j=0;j<lines; j++)
	{
		wmove(win, j, 0);
		whline(win, ' ', cols);
	}
	unsetcol(win);	
		

	int size = file_list.size();		
	
	
	const int maxlen = 12;
	
	// . 자동 컬럼 지정 모드
	// 1. 파일 개수에 맞게 컬럼수 지정 -> 모드 선택
	{
		if (size <= lines) ncol = 1;
		else if (size <= lines*2) ncol = 2;
		else if (size <= lines*3) ncol = 3;
		else if (size <= lines*4) ncol = 4;
		else if (size <= lines*5) ncol = 5;
		else ncol = 6;
		
		// 최대 가능 컬럼수  maxcol = cols / 12(최소 열두자) 		
		// 적어도 12자는 보이게 하자	
		
		if (ncol > cols / maxlen) ncol = cols / maxlen;				
		nrow = lines;	
		
		if ( (maxlen+39)*ncol-1 <= cols) drawmode1((cols+1)/ncol-39, bDrawCursor);
		else if ( (maxlen+24)*ncol-1 <= cols) drawmode2((cols+1)/ncol-24, bDrawCursor);
		else if ( (maxlen+9)*ncol-1 <= cols) drawmode3((cols+1)/ncol-9, bDrawCursor);
		else drawmode4((cols+1)/ncol-2, bDrawCursor);
	}
	
	// 2. 파일 최대 길이에 따른 모드 선택 -> 컬럼수 지정	
	/*
	{		
		int nMaxLen = (*max_element(file_list.begin(), file_list.end(), StringLengthCompare()))->name.size();
		
		if (size >= lines) ncol = 1;
		else ncol = lines / size;
		nrow = lines;
		
		if (ncol > cols / nMaxLen) ncol = cols / nMaxLen;
				
		if ( (nMaxLen+39)*ncol-1 <= cols) drawmode1((cols+1)/ncol-39);
		else if ( (nMaxLen+24)*ncol-1 <= cols) drawmode2((cols+1)/ncol-24);
		else if ( (nMaxLen+9)*ncol-1 <= cols) drawmode3((cols+1)/ncol-9);
		else drawmode4(nMaxLen);
	}
	*/
	
	// 3. 아무것도 없는 배째라 모드(모드를 중심으로 선택)

	// . 화면에 출력할 파일리스트 컬럼을 결정한다.
	/*
	if (conf.AutoDrawmode)
	{
		if (size < lines) conf.column = 1;
		else if (size < (lines)*2) conf.column = 2;
		else if (size < (lines)*3) conf.column = 3;
		else if (size < (lines)*4) conf.column = 4;
		else if (size < (lines)*5) conf.column = 5;
		else conf.column = 6;
	}	
	
	// 줄수 조정	
	if (conf.FixedRow)
	{
		nrow=lines;
	}
	else
	{
		nrow = ((size+conf.column-1) / conf.column);
		if (nrow > lines - 6) nrow = lines-6;
		if (lines - 6 < 0 ) nrow = 0;
	}
	*/
	
		
	
/*
	// . 파일 출력하기..
	switch(conf.column)
	{
	case 1:
		drawfile1();
		break;
	case 2:
		drawfile2();
		break;
	case 3:
		drawfile3();
		break;
	case 4:
		drawfile4();
		break;

	case 5:
		drawfile5();
		break;

	case 6:
		drawfile6();
		break;
	}
	*/
	redrawwin(win);
	wrefresh(win);
	
	// 찾기 창
	setcol(e_tColor.Default);
		if (!strSearch.empty())
		mvprintw(2, COLS-22, "[%-20s]", strSearch.c_str());
	unsetcol();
	
	
}


void Panel::DrawStatus()
{
	//  FileManager Status	
	setcol(e_tColor.StatA);
	move(LINES-2, 0);
	hline(' ', COLS);
	
	mvprintw(LINES-2, 0, "%5d ", nfile);
	mvprintw(LINES-2, 10, "%5d ", ndir);
	mvprintw(LINES-2, 23, "%14.14s ", toregular(dir_size).c_str());
	
	if (sel_num)
	{
		 mvprintw(LINES-2, 42, "%5d", sel_num);
		 if (sel_size >= 1000000000)
		 	mvprintw(LINES-2, 58, " %10.3f G", (float)sel_size/1073741824);
		 else
		 	mvprintw(LINES-2, 58, " %11.11s ", toregular(sel_size).c_str());
	}
	else mvprintw(LINES-2, 42, "%42s", "");

	unsetcol();

	setcol(e_tColor.Stat);
	mvprintw(LINES-2, 6,	gettext("File"));
	mvprintw(LINES-2, 16, 	gettext("Dir"));
	mvprintw(LINES-2, 38, 	gettext("Byte"));

	if (sel_num)
	{
		mvprintw(LINES-2, 47, gettext(" Selected ("));
		mvprintw(LINES-2, 71, gettext("Bytes)       "));
	}
	unsetcol();

	setcol(COLOR_YELLOW+8, COLOR_WHITE);
	mvaddch(LINES-2, COLS-2, kindsort[e_tConf.sort%6]);
	unsetcol();
	
	setcol(COLOR_RED, COLOR_WHITE);	
	if(e_tConf.ShowHidden) mvaddch(LINES-2, COLS-1, 'H');
	unsetcol();

	// Functional Status Panel
	// attr
	// //소유자
	// //그룹
	// 만들어진 시간
	// 크기
	// 파일 이름
	
	File &rf = *file_list[cur];
	
	setcol(e_tColor.Func);
	
	move(LINES-1, 0);
	hline(' ', COLS);

	mvprintw(LINES-1, 0, "%-11.11s", rf.attr);
	mvprintw(LINES-1, 12, "%10.0d", rf.size);
	mvprintw(LINES-1, 23, "%s %s", rf.date, rf.time);
	mvprintw(LINES-1, 38, "%s", rf.name.substr(0, COLS-38).c_str());

	unsetcol();

	setcol(COLOR_BLACK, e_tColor.Func.back);
	mvaddch(LINES-1, 11, VLINE);
	mvaddch(LINES-1, 22, VLINE);
	mvaddch(LINES-1, 37, VLINE);
	unsetcol();
	
}

// 퍼미션(10) + 1 + 날짜(8) + 1 + 시간(5) + 셀렉트 표시(1) + 파일(n) + 1 + 크기(10) + 1
// = n + 38
void Panel::drawmode1(int n, bool bDrawCursor)
{
	int width = n + 39;
	int  page = cur / (ncol * nrow);
	uint cnum = page * ncol * nrow;
	
	pFileIterator p = file_list.begin() + cnum;
	
	// 경계선 긋기	
	setcol(e_tColor.Line, win);	
	for (int i=1; i<ncol; i++)
		for (int j=0; j<nrow; j++)
			mvwaddch (win, j, i * width - 1, VLINE);
	unsetcol(win);		

	for (int col=0; col<ncol; col++)
	for (int row=0; row<nrow; row++)
	{
		if (p == file_list.end()) return;
		File &rf = *(*p);
		
		wmove(win,row,col*width);
		
		// 디렉토리일 경우
		if (rf.bDir)
		{
			if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
			else setcol(e_tColor.Default, win);		
			
			whline(win,' ', width-1);
			wprintw(win,"%s %s %s", rf.attr, rf.date, rf.time);			
			unsetcol(win);
			
			if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, e_tColor.Dir.font, win);
			else setcol(COLOR_WHITE+8, e_tColor.Dir.back, win);				
			
			if (rf.selected) waddch(win, '*');
			else waddch(win, ' ');			
			unsetcol(win);			
			
			if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
			else setcol(e_tColor.Dir, win);

			if (rf.name.size() >= n) wprintw(win,"%s~ ", rf.name.substr(0, n-1).c_str());
			else wprintw(win,"%s ", rf.name.c_str());			
						
			mvwprintw(win,row, col*width+width-12, "[ SubDir ] ");
			unsetcol(win);
		}		
		else
		{
			if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
			else setcol(e_tColor.Default, win);
			
			whline(win,' ', width-1);
			wprintw(win,"%s %s %s",rf.attr, rf.date, rf.time);
			unsetcol(win);
			
			if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, rf.color->font, win);
			else setcol(COLOR_WHITE+8, rf.color->back, win);				
			
			if (rf.selected) waddch(win, '*');
			else waddch(win, ' ');
			
			unsetcol(win);			
			
			
			if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
			else setcol(*rf.color, win);			
			
			if (rf.name.size() > n ) wprintw(win,"%s~ ", rf.name.substr(0,n-1).c_str());
			else wprintw(win,"%s ", rf.name.c_str());		
			
			wmove(win, row, col * width + width -12);

			if (rf.size >= 10000000)
			{
				wprintw(win,"%9.1f", (float)rf.size/1048576);
				unsetcol(win);

				if (cnum == cur) setcol(11, rf.color->font, win);
				else setcol(11, e_tColor.Default.back, win);
				waddch(win,'M');
				waddch(win,' ');
				unsetcol(win);
			}
			else wprintw (win, "%10s ", toregular(rf.size).c_str());

			unsetcol(win);
		}

		p++;
		cnum++;
	}
	
}

// 파일(n) + 셀렉트 표시(1) + 크기(6) + 1 + 날짜(8) + 1 + 시간(5) + 1 
// = n + 23
void Panel::drawmode2(int n, bool bDrawCursor)
{
	int width = n + 24;
	int  page = cur / (ncol * nrow);
	uint cnum = page * ncol * nrow;
	
	pFileIterator p = file_list.begin() + cnum;
	
	// 경계선 긋기	
	setcol(e_tColor.Line, win);	
	for (int i=1; i<ncol; i++)
		for (int j=0; j<nrow; j++)
			mvwaddch (win, j, i * width - 1, VLINE);
	unsetcol(win);		

	for (int col=0; col<ncol; col++)
	for (int row=0; row<nrow; row++)
	{
		if (p == file_list.end()) return;
		
		File &rf = *(*p);

		if (rf.bDir)
		{
			if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
			else setcol(e_tColor.Dir, win);
			
			wmove(win,row, col*width);
			whline(win,' ', width-1);

			if (rf.name.size() >= n ) mvwprintw(win,row,col*width, "%s~", rf.name.substr(0, n-1).c_str());
			else mvwprintw(win,row,col*width, "%s", rf.name.c_str());
			
			unsetcol(win);

			if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, e_tColor.Dir.font, win);
			else setcol(COLOR_WHITE+8, e_tColor.Dir.back, win);
			
			wmove(win,row, col*width+width-24);	

			if (rf.selected) waddch(win, '*');
			else waddch(win, ' ');
			
			unsetcol(win);
			
			
			if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
			else setcol(e_tColor.Dir, win);		
			

			wprintw(win,"  DIR  ");
			unsetcol(win);

			if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
			else setcol(e_tColor.Default, win);


			wprintw(win,"%s %s ",rf.date, rf.time);
			unsetcol(win);
		}
		else
		{
			
			if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
			else setcol(*rf.color, win);
			
			wmove(win,row, col*width);
			whline(win,' ', width-1);
			
			if (rf.name.size() >= n) mvwprintw(win,row,col*width, "%s~", rf.name.substr(0, n-1).c_str());
			else mvwprintw(win,row,col*width, "%s", rf.name.c_str());
			
			unsetcol(win);
			
			wmove(win,row, col*width+width-24);
			
			if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, rf.color->font, win);
			else setcol(COLOR_WHITE+8, rf.color->back, win);			

			if (rf.selected) waddch(win,'*');
			else waddch(win,' ');
			
			unsetcol(win);			
			

			if (bDrawCursor && cnum == cur) setrcol(*rf.color,win);
			else setcol(e_tColor.Default, win);


			if (rf.size >= 1000000)
			{
				wprintw(win,"%5.1f", (float)rf.size/1048576);
				unsetcol(win);

				if (bDrawCursor && cnum == cur) setcol(11, rf.color->font, win);
				else setcol(11, e_tColor.Default.back, win);
				waddch(win,'M');

				unsetcol(win);

				if (bDrawCursor && cnum == cur) setrcol(*rf.color,win);
				else setcol(e_tColor.Default,win);


			}
			else wprintw (win,"%6d", rf.size);
			wprintw(win," %s %s ",rf.date, rf.time);

			unsetcol(win);

		}

		p++;
		cnum++;
	}
}

// 셀렉트 표시(1) + 파일(n) + 1 + 크기(6)
// = n + 8
void Panel::drawmode3(int n, bool bDrawCursor)
{
	int width = n + 9;	
	int  page = cur / (ncol * nrow);
	uint cnum = page * ncol * nrow;
	
	pFileIterator p = file_list.begin() + cnum;
	
	// 경계선 긋기	
	setcol(e_tColor.Line, win);	
	for (int i=1; i<ncol; i++)
		for (int j=0; j<nrow; j++)
			mvwaddch (win, j, i * width - 1, VLINE);
	unsetcol(win);		

	for (int col=0; col<ncol; col++)
		for (int row=0; row<nrow; row++)
		{
			if (p == file_list.end()) return;
			File &rf = *(*p);
			
			wmove(win,row, col*width);

			if (rf.bDir)
			{				
				if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, e_tColor.Dir.font, win);
				else setcol(COLOR_WHITE+8, e_tColor.Dir.back, win);
				// 뒷배경 채우기 
				whline(win,' ', width-1);				
				// 셀렉트 표시 하기
				if (rf.selected) waddch(win,'*');
				else waddch(win,' ');
				unsetcol(win);
				
				if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
				else setcol(e_tColor.Dir, win);				
				if (rf.name.size() > width-2) wprintw(win,"%s~", rf.name.substr(0, width - 3).c_str());
				else wprintw(win,"%s", rf.name.c_str());
				unsetcol(win);
			}
			else
			{				
				if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, rf.color->font, win);
				else setcol(COLOR_WHITE+8, rf.color->back, win);				
				// 뒷배경 채우기 				
				whline(win,' ', width-1);				
				// 셀렉트 표시 하기	
				if (rf.selected) waddch(win,'*');
				else waddch(win,' ');				
				unsetcol(win);
				
				// 파일 표시				
				if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
				else setcol(*rf.color, win);				
				if (rf.name.size() > n) wprintw(win,"%s~ ", rf.name.substr(0, n-1).c_str());
				else wprintw(win,"%s ", rf.name.c_str());
				unsetcol(win);				
				
				// 크기 표시			
				wmove(win,row, col*width+width-7);
				
				if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
				else setcol(e_tColor.Default, win);

				if (rf.size >= 1000000)
				{
					wprintw(win,"%5.1f", (float)rf.size/1048576);
					unsetcol(win);
					if (bDrawCursor && cnum == cur) setcol(11, rf.color->font, win);
					else setcol(11, e_tColor.Default.back, win);
					waddch(win, 'M');
				}
				else wprintw(win,"%6d", rf.size);
				unsetcol(win);				
			}

			p++;
			cnum++;
		}
}

// 셀렉트 표시(1) + 파일(n)
// = n + 1
void Panel::drawmode4(int n, bool bDrawCursor)
{
	int width = n + 2;	
	int  page = cur / (ncol * nrow);
	uint cnum = page * ncol * nrow;
	
	pFileIterator p = file_list.begin() + cnum;
	
	// 경계선 긋기	
	setcol(e_tColor.Line, win);	
	for (int i=1; i<ncol; i++)
		for (int j=0; j<nrow; j++)
			mvwaddch (win, j, i * width - 1, VLINE);
	unsetcol(win);	
	
	for (int col=0; col<ncol; col++)
		for (int row=0; row<nrow; row++)
		{
			if (p == file_list.end()) return;
			
			File &rf = *(*p);
			
			wmove(win,row, col*width);
			
			if (rf.bDir)
			{
				// 뒷 배경 채우기
				if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
				else setcol(e_tColor.Dir, win);	
				whline(win,' ', width-1);			
				unsetcol(win);
				
				// 셀렉트 표시 하기
				if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, e_tColor.Dir.font, win);
				else setcol(COLOR_WHITE+8, e_tColor.Dir.back, win);				
				if (rf.selected) waddch(win,'*');
				else waddch(win,' ');			
				unsetcol(win);
				
				// 디렉토리 이름 표시하기			
				if (bDrawCursor && cnum == cur) setrcol(e_tColor.Dir, win);
				else setcol(e_tColor.Dir, win);				
				if (rf.name.size() > width-2) wprintw(win,"%s~", rf.name.substr(0, width-3).c_str());
				else wprintw(win,"%s", rf.name.c_str());
				unsetcol(win);		
				
			}
			else
			{
				// 뒷배경 채우기 			
				if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
				else setcol(*rf.color, win);				
				whline(win,' ', width-1);
				unsetcol(win);
				
				// 셀렉트 표시 						
				if (bDrawCursor && cnum == cur) setcol(COLOR_WHITE+8, rf.color->font, win);
				else setcol(COLOR_WHITE+8, rf.color->back, win);				
				if (rf.selected) waddch(win,'*');
				else waddch(win,' ');			
				unsetcol(win);	
				
				// 파일 이름 표시
				if (bDrawCursor && cnum == cur) setrcol(*rf.color, win);
				else setcol(*rf.color, win);					
				if (rf.name.size() > n) wprintw(win,"%s~", rf.name.substr(0, n-1).c_str());
				else wprintw(win,"%s", rf.name.c_str());
				unsetcol(win);
			}

			p++;
			cnum++;
		}
}


///	\brief	1컬럼을 갖는 파일리스트를 화면에 출력한다.
void Panel::drawfile1(bool b)
{
	// 형식 1
	ncol=1;
	drawmode1(cols-39,b);

}

///	\brief	2컬럼을 갖는 파일리스트를 화면에 출력한다.
void Panel::drawfile2(bool b)
{
	ncol = 2;
	drawmode2(cols/2-24,b);
}

///	\brief	3컬럼을 갖는 파일리스트를 화면에 출력한다.
void Panel::drawfile3(bool b)
{
	ncol = 3;
	drawmode3(cols/3-9,b);
}


///	\brief	4컬럼을 갖는 파일리스트를 화면에 출력한다.
void Panel::drawfile4(bool b)
{
	ncol=4;
	drawmode3(cols/4-9,b);
}

///	\brief	5컬럼을 갖는 파일리스트를 화면에 출력한다.
void Panel::drawfile5(bool b)
{
	ncol = 5;	
	drawmode4(cols/5-2,b);	
}

///	\brief	6컬럼을 갖는 파일리스트를 화면에 출력한다.
void Panel::drawfile6(bool b)
{	
	ncol=6;
	drawmode4(cols/6-2, b);
}

