#include "commandimp.h"
#include <vector>
#include <deque>

namespace MLS {

using namespace std;

deque<string> g_CmdHistory;

vector<string> &PathComplete(const string &path)
{
	//          l
	// 012345678901
	// workspace/l
	// /usr/src/red
	static vector<string> entries;
	entries.clear();
	
	string::size_type loc;
	
	string d, name;
	
	if ((loc = path.rfind('/')) == string::npos)
	{
		// 지정 디렉토리가 없다면 현 디렉토리 및 path 검색		
		name = path;
		int len = name.size();
		
		DIR *dir = opendir(".");
		if (!dir) return entries;
			
		struct dirent *entry;
		while( (entry = readdir(dir)) != NULL)
		{
			if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) continue;
				
			if (!name.compare(0, len, entry->d_name, len))
			{
				string fullname = d + entry->d_name;
					
				struct stat statbuf;
				if (stat(fullname.c_str(), &statbuf) == -1) continue;
					
				if (S_ISDIR(statbuf.st_mode)) fullname += '/';
				entries.push_back(entry->d_name);
			}
		}				
		closedir(dir);
		
		// path 상의 실행가능 파일들에 대해 검색
		StringTokenizer st(getenv("PATH"), ":");				
		while(st.Next())
		{
			d = st.Get();
			
			if (d.empty()) continue;
			if (*d.rbegin() != '/') d += '/';			
						
			DIR *dir = opendir(d.c_str());		
			if (!dir) continue;
			
			struct dirent *entry;
			while( (entry = readdir(dir)) != NULL)
			{
				if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) continue;		
				
				if (!name.compare(0, len, entry->d_name, len))
				{
					string fullname = d + entry->d_name;
					
					struct stat statbuf;
					if (stat(fullname.c_str(), &statbuf) == -1) continue;
					
					if (S_ISDIR(statbuf.st_mode)) continue;
					
					if ((S_IXUSR & statbuf.st_mode) || (S_IXGRP & statbuf.st_mode) || (S_IXOTH & statbuf.st_mode))
						entries.push_back(entry->d_name);
				}
			}				
			closedir(dir);
		}
		
		
	}
	else
	{
		d = path.substr(0, loc+1);
		name = path.substr(loc+1);
		int len = name.size();
		
		// 지정 디렉토리가 있다면 				
		DIR *dir = opendir(d.c_str());		
		if (!dir) return entries;
		
		struct dirent *entry;		
		while( (entry = readdir(dir)) != NULL)
		{
			if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) continue;
		
			
			if (!name.compare(0, len, entry->d_name, len))
			{
				string fullname = d + entry->d_name;
				
				struct stat statbuf;
				if (stat(fullname.c_str(), &statbuf) == -1) continue;
				
				if (S_ISDIR(statbuf.st_mode)) fullname += '/';
				
				entries.push_back(fullname);
			}
		}				
		closedir(dir);		
	}
	
	return entries;
}

void CommandImp::Shell()
{ 
	string str;
	WINDOW *win = newwin(1, COLS, LINES-1, 0);	
	wbkgd(win, COLOR(COLOR_WHITE, COLOR_BLACK));
	
	curs_set(1);
	
	int W = COLS - 4;
	int cur=str.size(), scroll= str.size() > W ? str.size()-W : 0; // 커서, 스크롤	
	int key;
	
	vector<string> com_entry;	
	int index = 0, start = 0;
	int his = g_CmdHistory.size();
	
	if (panel->isZip() == true)
	{
		MsgBox(1, gettext("Error"), gettext("Shell can't execute, zip exit !!!"));
		return;
	}
	
	g_MainFrame->bShell = true;
	
	while(1)
	{
	// 찍고
		wattron (win, COLOR(COLOR_WHITE, COLOR_BLACK));
		wmove(win, 0, 0);
		whline(win, ' ', COLS);
		wprintw(win, "$ ");
		wprintw(win, "%s", str.substr(scroll, W).c_str());		
	
	// 커서가 있는 곳으로 이동한다.
		wmove(win, 0, 2+cur-scroll);
		wrefresh(win);
	
		key = getch();
		
		if (key == '\t')
		{
			//      l            c
			// 01234567890123456789
			// break /usr/src/red p
			// /usr/src/redhat
			// 커서를 기점으로 앞의 것을 취한다.
			
			if (!index)
			{
				string::size_type loc;
				if ((loc=str.rfind(' ', cur-1)) != string::npos)
					start = loc + 1;
				else
					start = 0;
				
				com_entry = PathComplete(str.substr(start, cur-start));
			}
			
			if (com_entry.empty()) continue;

			string &rf = com_entry[index % com_entry.size()];
			
			str.erase(start, cur-start);
			str.insert(start, rf);
			cur = start + rf.size();
			if (cur - scroll >= W) scroll = cur - W;
			
			index++;
		}
		else index = 0;	
		
		if (key == KEY_UP)
		{
			if (g_CmdHistory.empty()) continue;
			
			his--;			
			if (his < 0) his = g_CmdHistory.size()-1;
			
			str = g_CmdHistory[his];
			cur = str.size();
			if (cur - scroll >= W) scroll = cur - W ;
		}
				
		if (key == KEY_DOWN)
		{
			if (g_CmdHistory.empty()) continue;
			
			his++;
			if (his >= g_CmdHistory.size()) his = 0;
			
			str = g_CmdHistory[his];
			cur = str.size();
			if (cur - scroll >= W) scroll = cur - W ;
		}		
	
	// home, end, pgup, pgdn, 화살표, 아스키
	// 기타 키가 들어오면 씹는다..
		if (key == KEY_HOME || key == KEY_PPAGE)
		{
			cur = 0;
			scroll = 0;
		}
	
		if (key == KEY_END || key == KEY_NPAGE)  
		{
			cur = str.size();
			if (cur - scroll >= W) scroll = cur - W ;
		}
	
		if (key == KEY_LEFT)
		{
			cur--;
			if (cur < 0) cur = 0;
			if (cur - scroll < 0) scroll = cur;
		}
	
		if (key == KEY_RIGHT)
		{
			cur++;
			if (cur > str.size()) cur = str.size();
			if (cur - scroll >= W) scroll = cur - W ;
		}
					
		if (key == 27) 
		{
			g_MainFrame->bShell = false;
			curs_set(0);
			delwin(win);
			Refresh();
			return;
		}
		
		if (key == KEY_RESIZE)
		{
			curs_set(0);
			delwin(win);
			Refresh();
			return;
		}
		
		if (key == 13) break;
		
	// backspace 커서 위치에서 한글자 지운다.
		if (key == 127)
		{
			if (!str.empty() && cur)	
			{
				str.erase(cur-1, 1);
				cur--;
				if (cur - scroll < 0) scroll = cur;
			}			
			continue;
		}
	
	// del..		
		if (key == 330)
		{
			if (!str.empty())
				str.erase(cur, 1);
			continue;
		}
	
		if (32 <= key && key <= 127)
		{
		//MsgBox(1, "f", "%d", cur);
			str.insert(cur, 1, (char) key);
			cur++;
			if (cur - scroll >= W) scroll = cur - W;
			continue;
		}
	}	
	
	curs_set(0);
	delwin(win);
	
	if (str.empty()) 
	{
		g_MainFrame->bShell = false;
		return;
	}
	
	while(g_CmdHistory.size() >= 50) g_CmdHistory.pop_front();	
	g_CmdHistory.push_back(str);
	
	ParseAndRun(str, true);
	
	Refresh();
}

};

